/*
 */

#include "ErrorM.h"

#include <stdio.h>
#include <stdarg.h>
#include <string>


std::string vstringf(const char* fmt, va_list va)
{  // BSD and compatible environments only:
   size_t len = vsnprintf(NULL, 0, fmt, va);
   char* cp = new char[len+1]; // political correct (do not write the internal string data structures directly)
   vsnprintf(cp, len+1, fmt, va);
   std::string s(cp, len);
   delete[] cp;
   return s;
}

std::string stringf(const char* fmt, ...)
{  va_list va;
   va_start(va, fmt);
   const std::string& s = vstringf(fmt, va);
   va_end(va);
   return s;
}


const char* Class2String(EventClass cls)
{  static const char c2s[][8] =
   {  "Debug",
      "Success",
      "Info",
      "Warning",
      "ERROR",
      "FATAL",
      "unknown"
   };
   return c2s[(unsigned)cls > Fatal ? Fatal : cls];
}


Event::Event(EventClass cls, int id, const char* msg, ...)
{  Class = cls;
   ID = id;
   va_list va;
   va_start(va, msg);
   Message = vstringf(msg, va);
   va_end(va);
}

Event::operator std::string() const
{  std::string s(Class2String(Class));
   return s.append(8-s.size(), ' ') + Message;
}

void AEventHandler::PostEvent(const Event& e)
{  (Event&)*this = e; // slicing
   if (e.Class >= Error)
      throw *this;
}

void EventLog::PostEvent(const Event& e)
{  AEventHandler::PostEvent(e);
   *Log << (std::string)e << std::endl;
}


static AEventHandler noEH;

AEventHandler* DefEH = &noEH;

