/*
 *  bitstream.h - bitstream class
 *
 *  Copyright (C) 2003 Marcel Mueller
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "bitstream.h"

#include <assert.h>


// bitstream class
const unsigned char bitstream::bitmask[9] = {0, 1, 3, 7, 15, 31, 63, 127, 255};

unsigned int bitstream::get(unsigned int bits)
{  assert(bits <= 8);
   unsigned int rval = (sibuf[0] << 8) | sibuf[1];
   bit += bits;
   rval >>= 16-bit;
   sibuf += (bit>>3);
   bit &= 7;
   return rval & bitmask[bits];
}

unsigned int bitstream::peek(unsigned int bits)
{  assert(bits <= 8);
   unsigned int rval = (sibuf[0] << 8) | sibuf[1];
   rval >>= 16-bit-bits;
   return rval & bitmask[bits];
}

void bitstream::skip(unsigned int bits)
{  bit += bits;
   sibuf += (bit>>3);
   bit &= 7;
}

void bitstream::set(unsigned int bits, unsigned int value)
{  assert(bits <= 8);
   unsigned int rval = (sibuf[0] << 8) | sibuf[1];
   bit += bits;
   unsigned int shift = 16-bit;
   rval = (rval & ~(bitmask[bits] << shift)) | value << shift;
   sibuf[1] = (unsigned char)rval; // write back
   sibuf[0] = (unsigned char)(rval >> 8);
   sibuf += (bit>>3);
   bit &= 7;
}

