/*
 *  filechange.h - analyzes mp3 files, determines the perceived volume, modify the MPEG files
 *
 *  Copyright (C) 2002-2004 Glen Sawyer & Marcel Mueller
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef filechange_h
#define filechange_h

#include "MPEGstream.h"
#include "gain_analysis.h"


class fileinfo
{public: // global options
   static bool    databaseFormat;
   static bool    QuietMode;
   static int     ForceLayer;

 public: // results: read only, please ...
   // source info
   const char* const filename;
   bool           fileOK;
   // analysis results
   int            layer;
   Float_t        maxsample;
   double         dBchange;

 protected: // internal funcs
   static double  fromdB(double dB)          { return pow(10, dB/20.); }
   int            intgainchange() const      { return (int)floor(MPEGstream::factor2int(layer, fromdB(dBchange)) + .5); }
   virtual int    passError(int lerrnum, int numStrings, ...); // returns lerrnum

 public:
   // constructor
   fileinfo(const char* f) : filename(f), fileOK(false) {}
   // public functions
   virtual int    Analyze(); // analyze MPEG file
   virtual void   PrintGainInfo() const;
};


class filechange : public fileinfo
{public:
   // global options
   static bool    ignoreClipWarning;
   static bool    InteractiveMode;
   static const char* defInput;
   static bool    autoNoClip;
   static bool    wrapGain;
   static bool    UsingTemp;
   static bool    saveTime;

 public:
   // constructor
   filechange(const char* f) : fileinfo(f) {}
   // public functions
   virtual int    CommitGainChange() const;
   virtual int    ChangeGain(int leftgainchange, int rightgainchange);
           int    ChangeGain(int gainchange)
                  {  return ChangeGain(gainchange, gainchange); }
};


#endif

